@@ -13,7 +13,7 @@
 
 package IO::Socket::SSL;
 
-our $VERSION = '2.015';
+our $VERSION = '2.016';
 
 use IO::Socket;
 use Net::SSLeay 1.46;
@@ -230,6 +230,10 @@ if (!defined &Net::SSLeay::CTX_clear_options) {
     };
 }
 
+# Try to work around problems with alternative trust path by default, RT#104759
+my $DEFAULT_X509_STORE_flags = 0;
+eval { $DEFAULT_X509_STORE_flags |= Net::SSLeay::X509_V_FLAG_TRUSTED_FIRST() };
+
 our $DEBUG;
 use vars qw(@ISA $SSL_ERROR @EXPORT);
 
@@ -256,6 +260,7 @@ use vars qw(@ISA $SSL_ERROR @EXPORT);
 my @caller_force_inet4; # in case inet4 gets forced we store here who forced it
 
 my $IOCLASS;
+my $family_key; # 'Domain'||'Family'
 BEGIN {
     # declare @ISA depending of the installed socket class
 
@@ -275,6 +280,7 @@ BEGIN {
     };
 
     # try IO::Socket::IP or IO::Socket::INET6 for IPv6 support
+    $family_key = 'Domain'; # traditional
     if ( $ip6 ) {
 	# if we have IO::Socket::IP >= 0.31 we will use this in preference
 	# because it can handle both IPv4 and IPv6
@@ -284,6 +290,7 @@ BEGIN {
 	}) {
 	    @ISA = qw(IO::Socket::IP);
 	    constant->import( CAN_IPV6 => "IO::Socket::IP" );
+	    $family_key = 'Family';
 	    $IOCLASS = "IO::Socket::IP";
 
 	# if we have IO::Socket::INET6 we will use this not IO::Socket::INET
@@ -315,7 +322,12 @@ BEGIN {
 
 sub DEBUG {
     $DEBUG or return;
-    my (undef,$file,$line) = caller;
+    my (undef,$file,$line,$sub) = caller(1);
+    if ($sub =~m{^IO::Socket::SSL::(?:error|(_internal_error))$}) {
+	(undef,$file,$line) = caller(2) if $1;
+    } else {
+	(undef,$file,$line) = caller;
+    }
     my $msg = shift;
     $file = '...'.substr( $file,-17 ) if length($file)>20;
     $msg = sprintf $msg,@_ if @_;
@@ -419,7 +431,7 @@ sub import {
 		    @ISA = ( CAN_IPV6 );
 		    warn "IPv6 support re-enabled in __PACKAGE__, got disabled in file $caller_force_inet4[1] line $caller_force_inet4[2]";
 		} else {
-		    die "INET6 is not supported, install IO::Socket::INET6";
+		    die "INET6 is not supported, install IO::Socket::IP";
 		}
 	    }
 	} elsif ( /^:?debug(\d+)/ ) {
@@ -468,6 +480,7 @@ sub configure {
 
     $self->configure_SSL($arg_hash) || return;
 
+    $arg_hash->{$family_key} ||= $arg_hash->{Domain} || $arg_hash->{Family};
     return $self->_internal_error("@ISA configuration failed",0)
 	if ! $self->SUPER::configure($arg_hash);
 
@@ -668,12 +681,12 @@ sub connect_SSL {
 
     my $start = defined($timeout) && time();
     {
-	#DEBUG( 'calling ssleay::connect' );
 	$SSL_ERROR = undef;
 	$CURRENT_SSL_OBJECT = $self;
+	$DEBUG>=3 && DEBUG("call Net::SSLeay::connect" );
 	my $rv = Net::SSLeay::connect($ssl);
 	$CURRENT_SSL_OBJECT = undef;
-	$DEBUG>=3 && DEBUG("Net::SSLeay::connect -> $rv" );
+	$DEBUG>=3 && DEBUG("done Net::SSLeay::connect -> $rv" );
 	if ( $rv < 0 ) {
 	    if ( my $err = $self->_skip_rw_error( $ssl,$rv )) {
 		$self->error("SSL connect attempt failed");
@@ -826,7 +839,7 @@ sub accept {
 	    };
 	    $DEBUG>=2 && DEBUG('will not start SSL handshake yet');
 	    return wantarray ? ($socket, getpeername($socket) ) : $socket
-	};
+	}
     }
 
     $self->accept_SSL($socket) || return;
@@ -1160,7 +1173,7 @@ sub readline {
 	    $buf .= $pb
 	} else {
 	    return $buf eq '' ? ():$buf;
-	};
+	}
 	if ( !$eod ) {
 	    my $pos = index( $buf,$delim0 );
 	    if ( $pos<0 ) {
@@ -2311,11 +2324,9 @@ WARN
 	    }
 	}
 
+	my $X509_STORE_flags = $DEFAULT_X509_STORE_flags;
 	if ($arg_hash->{'SSL_check_crl'}) {
-	    Net::SSLeay::X509_STORE_set_flags(
-		Net::SSLeay::CTX_get_cert_store($ctx),
-		Net::SSLeay::X509_V_FLAG_CRL_CHECK()
-	    );
+	    $X509_STORE_flags |= Net::SSLeay::X509_V_FLAG_CRL_CHECK();
 	    if ($arg_hash->{'SSL_crl_file'}) {
 		my $bio = Net::SSLeay::BIO_new_file($arg_hash->{'SSL_crl_file'}, 'r');
 		my $crl = Net::SSLeay::PEM_read_bio_X509_CRL($bio);
@@ -2327,6 +2338,11 @@ WARN
 	    }
 	}
 
+	Net::SSLeay::X509_STORE_set_flags(
+	    Net::SSLeay::CTX_get_cert_store($ctx),
+	    $X509_STORE_flags
+	) if $X509_STORE_flags;
+
 	Net::SSLeay::CTX_set_default_passwd_cb($ctx,$arg_hash->{SSL_passwd_cb})
 	    if $arg_hash->{SSL_passwd_cb};
 
@@ -2373,8 +2389,8 @@ WARN
 		# don't free @chain, because CTX_add_extra_chain_cert
 		# did not duplicate the certificates
 	    }
-	    $havecert or return
-		IO::Socket::SSL->error("Failed to use certificate file");
+	    $havecert or return IO::Socket::SSL->error(
+		"Failed to load certificate from file (no PEM, DER or PKCS12)");
 	}
 
 	if (!$havecert || $havekey) {
@@ -2392,9 +2408,8 @@ WARN
 		    last;
 		}
 	    }
-	}
-	if ($havecert && !$havekey) {
-	    return IO::Socket::SSL->error("Failed to use private key");
+	    $havekey or return IO::Socket::SSL->error(
+		"Failed to load key from file (no PEM or DER)");
 	}
 
 	# replace arg_hash with created context
